"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LANSession = exports.DiscoveryLAN = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const ya_dns_sd_1 = require("ya-dns-sd");
const deps_js_1 = require("../../deps.js");
const errors_js_1 = require("../util/errors.js");
const partner_tcp_js_1 = require("../syncer/partner_tcp.js");
const message_crypto_js_1 = require("../syncer/message_crypto.js");
const misc_js_1 = require("../util/misc.js");
const tcp_provider_node_js_1 = require("../tcp/tcp_provider.node.js");
const ES_PORT = 17171;
/** A discovery service for finding peers on the local network, to be used with `Peer.discover`.
 */
class DiscoveryLAN {
    constructor(opts) {
        Object.defineProperty(this, "abortController", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new AbortController()
        });
        Object.defineProperty(this, "multicastInterface", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new ya_dns_sd_1.MulticastInterface()
        });
        Object.defineProperty(this, "tcpProvider", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new tcp_provider_node_js_1.TcpProvider()
        });
        //  IP address.
        Object.defineProperty(this, "sessions", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        // A map of IP addresses to service names.
        Object.defineProperty(this, "serviceNames", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "eventQueue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new deps_js_1.AsyncQueue()
        });
        // Need to set up a listener here that can create partners...
        const listener = this.tcpProvider.listen({ port: opts.port || ES_PORT });
        const shouldAdvertise = opts.advertise === undefined
            ? true
            : opts.advertise;
        if (shouldAdvertise) {
            (0, ya_dns_sd_1.advertise)({
                multicastInterface: this.multicastInterface,
                service: {
                    name: opts.name,
                    port: opts.port || ES_PORT,
                    protocol: "tcp",
                    txt: {
                        version: new dntShim.TextEncoder().encode("10"),
                    },
                    type: "earthstar",
                },
                signal: this.abortController.signal,
            });
        }
        (async () => {
            for await (const conn of listener) {
                const key = `${conn.remoteAddr.hostname}`;
                const existingSession = this.sessions.get(key);
                if (existingSession) {
                    existingSession.addConn(conn);
                }
                else {
                    this.eventQueue.push({
                        kind: "PEER_INITIATED_SYNC",
                        description: this.serviceNames.get(key) || key,
                        begin: async (peer) => {
                            const session = new LANSession({
                                onComplete: () => {
                                    this.sessions.delete(key);
                                    this.serviceNames.delete(key);
                                },
                                target: {
                                    hostname: conn.remoteAddr.hostname,
                                    name: this.serviceNames.get(key),
                                },
                                ourPort: opts.port || ES_PORT,
                            });
                            this.sessions.set(key, session);
                            session.addConn(conn);
                            const partner = await session.partner;
                            const syncer = peer.addSyncPartner(partner, this.serviceNames.get(key) || key);
                            session.syncer.resolve(syncer);
                            return syncer;
                        },
                    });
                }
            }
        })();
        (async () => {
            for await (const service of (0, ya_dns_sd_1.browse)({
                multicastInterface: this.multicastInterface,
                signal: this.abortController.signal,
                service: {
                    protocol: "tcp",
                    type: "earthstar",
                },
            })) {
                const key = `${service.host}`;
                this.serviceNames.set(key, service.name);
                if (service.isActive === false) {
                    this.eventQueue.push({
                        kind: "PEER_EXITED",
                        description: service.name,
                    });
                    continue;
                }
                this.eventQueue.push({
                    kind: "PEER_DISCOVERED",
                    description: service.name,
                    begin: async (peer, appetite) => {
                        if (this.sessions.has(key)) {
                            throw new Error("A syncer for this peer has already been started");
                        }
                        await (0, misc_js_1.sleep)(Math.random() * (120 - 20) + 20);
                        const session = new LANSession({
                            initiator: { appetite, port: service.port },
                            onComplete: () => {
                                this.sessions.delete(key);
                                this.serviceNames.delete(key);
                            },
                            target: {
                                hostname: service.host,
                                name: service.name,
                            },
                            ourPort: opts.port || ES_PORT,
                        });
                        this.sessions.set(key, session);
                        const partner = await session.partner;
                        const syncer = peer.addSyncPartner(partner, this.serviceNames.get(key) || key);
                        session.syncer.resolve(syncer);
                        return syncer;
                    },
                });
            }
        })();
    }
    get events() {
        return this.eventQueue;
    }
    stop() {
        this.abortController.abort();
        this.eventQueue.push({
            kind: "SERVICE_STOPPED",
        });
        this.eventQueue.close();
    }
}
exports.DiscoveryLAN = DiscoveryLAN;
class LANSession {
    constructor(opts) {
        Object.defineProperty(this, "initiator", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "description", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "tcpProvider", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new tcp_provider_node_js_1.TcpProvider()
        });
        // ECDH
        Object.defineProperty(this, "keypair", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        Object.defineProperty(this, "derivedSecret", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        //
        Object.defineProperty(this, "onComplete", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "targetPort", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        Object.defineProperty(this, "ourPort", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "partner", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        Object.defineProperty(this, "syncer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (0, deps_js_1.deferred)()
        });
        this.initiator = opts.initiator;
        this.ourPort = opts.ourPort;
        this.description = opts.target.name || `${opts.target.hostname}`;
        this.onComplete = opts.onComplete;
        dntShim.crypto.subtle.generateKey({
            name: "ECDH",
            namedCurve: "P-256",
        }, true, ["deriveKey", "deriveBits"]).then((keypair) => {
            this.keypair.resolve(keypair);
        });
        if (opts.initiator) {
            this.targetPort.resolve(opts.initiator?.port);
            // Open keyexchange connection
            this.tcpProvider.connect({
                hostname: opts.target.hostname,
                port: opts.initiator.port,
            }).then((conn) => {
                this.addKeyExchangeConn(conn);
            });
            // Open messaging connection
            this.tcpProvider.connect({
                hostname: opts.target.hostname,
                port: opts.initiator.port,
            }).then((conn) => {
                this.addMessageConn(conn);
            });
        }
        this.syncer.then((syncer) => {
            syncer.isDone().then(() => {
                this.onComplete();
            }).catch(() => {
                this.onComplete();
            });
        });
    }
    async addConn(conn) {
        const connKind = await identifyConn(conn);
        if (this.initiator && connKind !== ConnKind.Attachment) {
            // Only the initiator can start initiate messaging and key exchange.
            conn.close();
            return;
        }
        switch (connKind) {
            case ConnKind.Messages: {
                this.addMessageConn(conn);
                break;
            }
            case ConnKind.Attachment: {
                this.addAttachmentConn(conn);
                break;
            }
            case ConnKind.KeyExchange: {
                this.addKeyExchangeConn(conn);
            }
        }
    }
    async addKeyExchangeConn(conn) {
        const ourKeypair = await this.keypair;
        if (this.initiator) {
            const idByte = new Uint8Array(1);
            const idView = new DataView(idByte.buffer);
            idView.setUint8(0, ConnKind.KeyExchange);
            await conn.write(idByte);
        }
        // Send our key to them.
        const publicKeyExported = await dntShim.crypto.subtle.exportKey("raw", ourKeypair.publicKey);
        const keyExchangeMessageBytes = new Uint8Array(publicKeyExported);
        await conn.write(keyExchangeMessageBytes);
        const pubkeyBytes = new Uint8Array(65);
        const bytesRead = await conn.read(pubkeyBytes);
        conn.close();
        if (bytesRead === null) {
            throw new errors_js_1.ValidationError("Was not able to read bytes of public key");
        }
        const otherPublicKey = await dntShim.crypto.subtle.importKey("raw", pubkeyBytes, {
            name: "ECDH",
            namedCurve: "P-256",
        }, true, []);
        // set that to our other public key.
        const derivedKey = await dntShim.crypto.subtle.deriveKey({ name: "ECDH", public: otherPublicKey }, ourKeypair.privateKey, { name: "AES-GCM", length: 256 }, true, ["encrypt", "decrypt"]);
        this.derivedSecret.resolve(derivedKey);
    }
    async addMessageConn(conn) {
        if (this.syncer.state === "fulfilled") {
            return;
        }
        let appetite = this.initiator?.appetite;
        // If we're the initiator, send the identifying byte.
        if (this.initiator) {
            const idByte = new Uint8Array(1);
            const idView = new DataView(idByte.buffer);
            idView.setUint8(0, ConnKind.Messages);
            await conn.write(idByte);
            // and don't forget the appetite byte
            const appetiteByte = new Uint8Array(1);
            const appetiteView = new DataView(appetiteByte.buffer);
            appetiteView.setUint8(0, this.initiator.appetite === "once" ? 0 : 1);
            await conn.write(appetiteByte);
            // And don't forget the port byte.
            const portBytes = new Uint8Array(2);
            const portView = new DataView(portBytes.buffer);
            portView.setUint16(0, this.ourPort);
            await conn.write(portBytes);
        }
        if (!appetite) {
            // Read the conn for the appetite byte...
            const appetiteByte = new Uint8Array(1);
            await conn.read(appetiteByte);
            const appetiteView = new DataView(appetiteByte.buffer);
            appetite = appetiteView.getUint8(0) === 0 ? "once" : "continuous";
        }
        if (!this.initiator) {
            const portBytes = new Uint8Array(2);
            await conn.read(portBytes);
            const portView = new DataView(portBytes.buffer);
            this.targetPort.resolve(portView.getUint16(0));
        }
        // Create a new TCP partner here.
        const derivedKey = await this.derivedSecret;
        const partner = new partner_tcp_js_1.PartnerTcp(conn, appetite, derivedKey, await this.targetPort);
        this.partner.resolve(partner);
    }
    async addAttachmentConn(conn) {
        const derivedSecret = await this.derivedSecret;
        const transferDetails = (0, deps_js_1.deferred)();
        conn.readable
            .pipeThrough(new message_crypto_js_1.DecryptLengthDelimitStream(derivedSecret))
            .pipeThrough(new message_crypto_js_1.DecryptStream(derivedSecret))
            .pipeTo(new dntShim.WritableStream({
            write(chunk) {
                // We are only expecting one chunk.
                if (transferDetails.state !== "pending") {
                    return;
                }
                const authorBytes = chunk.subarray(0, 59);
                const decoder = new dntShim.TextDecoder();
                const shareLengthBytes = chunk.subarray(59, 60);
                const shareLenView = new DataView(shareLengthBytes.buffer, shareLengthBytes.byteOffset);
                const shareLength = shareLenView.getUint8(0);
                const shareAddressBytes = chunk.subarray(60, 60 + shareLength);
                const formatLengthBytes = chunk.subarray(60 + shareLength, 1);
                const formatLenView = new DataView(formatLengthBytes.buffer, formatLengthBytes.byteOffset);
                const formatLength = formatLenView.getUint8(0);
                const formatBytes = chunk.subarray(60 + shareLength + 1, 60 + shareLength + 1 + formatLength);
                const pathBytes = chunk.subarray(60 + shareLength + 1 + formatLength);
                const author = decoder.decode(authorBytes);
                const shareAddress = decoder.decode(shareAddressBytes);
                const formatName = decoder.decode(formatBytes);
                const path = decoder.decode(pathBytes);
                transferDetails.resolve({
                    author,
                    shareAddress,
                    formatName,
                    path,
                });
            },
        })).catch(() => {
            // At some point the other side will close the connection and this will throw.
        });
        const { author, path, formatName, shareAddress } = await transferDetails;
        const syncer = await this.syncer;
        await syncer.handleTransferRequest({
            kind: "download",
            author,
            path,
            formatName,
            shareAddress,
            source: conn,
        });
    }
}
exports.LANSession = LANSession;
var ConnKind;
(function (ConnKind) {
    ConnKind[ConnKind["Messages"] = 0] = "Messages";
    ConnKind[ConnKind["Attachment"] = 1] = "Attachment";
    ConnKind[ConnKind["KeyExchange"] = 2] = "KeyExchange";
})(ConnKind || (ConnKind = {}));
async function identifyConn(conn) {
    const firstByte = new Uint8Array(1);
    await conn.read(firstByte);
    const dataView = new DataView(firstByte.buffer);
    const kindId = dataView.getUint8(0);
    if (kindId > 2) {
        throw new errors_js_1.ValidationError("Incoming connection was not of valid type");
    }
    return kindId;
}
