"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecryptLengthDelimitStream = exports.DecryptStream = void 0;
const dntShim = __importStar(require("../../_dnt.shims.js"));
const deps_js_1 = require("../../deps.js");
/** Decrypts discrete encrypted chunks prefixed with a 12 byte initialisation value.  */
class DecryptStream {
    constructor(derivedKey) {
        Object.defineProperty(this, "transform", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "readable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.transform = new dntShim.TransformStream({
            async transform(chunk, controller) {
                if (chunk.byteLength === 0) {
                    return;
                }
                const decrypted = await dntShim.crypto.subtle.decrypt({
                    name: "AES-GCM",
                    iv: chunk.subarray(0, 12),
                }, derivedKey, chunk.subarray(12, undefined));
                controller.enqueue(new Uint8Array(decrypted));
            },
        });
        this.readable = this.transform.readable;
        this.writable = this.transform.writable;
    }
}
exports.DecryptStream = DecryptStream;
function isUnfinishedLength(o) {
    if (o === null) {
        return false;
    }
    return "unfinishedLengthBytes" in o;
}
function isUnfinishedMessage(o) {
    if (o === null) {
        return false;
    }
    return "unfinishedBytes" in o;
}
/** Given a stream of potentially multiple and / or truncated chunks with an encrypted length delimiter, finds encrypted length delimiters and uses that to enqueue discrete chunks */
class DecryptLengthDelimitStream {
    constructor(derivedKey) {
        Object.defineProperty(this, "transform", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "readable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "carryOver", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "derivedKey", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.derivedKey = derivedKey;
        const getCarryOver = this.getCarryOver.bind(this);
        const setCarryOver = this.setCarryOver.bind(this);
        const delimit = this.delimit.bind(this);
        this.transform = new dntShim.TransformStream({
            async transform(chunk, controller) {
                const carryOver = getCarryOver();
                setCarryOver(null);
                await delimit(chunk, carryOver, (bytes) => controller.enqueue(bytes));
            },
        });
        this.readable = this.transform.readable;
        this.writable = this.transform.writable;
    }
    getCarryOver() {
        return this.carryOver;
    }
    setCarryOver(carryOver) {
        this.carryOver = carryOver;
    }
    async delimit(newBytes, carryOver, enqueue) {
        let desiredLength;
        let bytesToWorkWith;
        if (isUnfinishedLength(carryOver)) {
            const together = (0, deps_js_1.concat)(carryOver.unfinishedLengthBytes, newBytes);
            const lengthResult = await this.getLength(together);
            if (lengthResult === false) {
                this.setCarryOver({
                    unfinishedLengthBytes: together,
                });
                return;
            }
            this.setCarryOver(null);
            desiredLength = lengthResult.length;
            bytesToWorkWith = lengthResult.leftover;
        }
        else if (isUnfinishedMessage(carryOver)) {
            desiredLength = carryOver.remaining;
            bytesToWorkWith = (0, deps_js_1.concat)(carryOver.unfinishedBytes, newBytes);
        }
        else {
            const lengthResult = await this.getLength(newBytes);
            if (lengthResult === false) {
                this.setCarryOver({
                    unfinishedLengthBytes: newBytes,
                });
                return;
            }
            this.setCarryOver(null);
            desiredLength = lengthResult.length;
            bytesToWorkWith = lengthResult.leftover;
        }
        if (desiredLength > bytesToWorkWith.byteLength) {
            this.setCarryOver({
                unfinishedBytes: bytesToWorkWith,
                remaining: desiredLength,
            });
            return;
        }
        else {
            const desiredBytes = bytesToWorkWith.subarray(0, desiredLength);
            const remaining = bytesToWorkWith.subarray(desiredLength);
            enqueue(desiredBytes);
            if (remaining.byteLength > 0) {
                await this.delimit(remaining, null, enqueue);
            }
        }
    }
    async getLength(bytes) {
        // Length of IV and tagged encrypted length
        if (bytes.byteLength < 12 + 20) {
            return false;
        }
        const decrypted = await dntShim.crypto.subtle.decrypt({
            name: "AES-GCM",
            iv: bytes.subarray(0, 12),
        }, this.derivedKey, bytes.subarray(12, 12 + 20));
        const view = new DataView(decrypted);
        const length = view.getUint32(0);
        // console.log("decrypted length", length);
        return {
            length,
            leftover: bytes.subarray(12 + 20),
        };
    }
}
exports.DecryptLengthDelimitStream = DecryptLengthDelimitStream;
