"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpListener = exports.TcpProvider = void 0;
/// <reference types="@types/node" />
const dntShim = __importStar(require("../../_dnt.shims.js"));
const net_1 = require("net");
const deps_js_1 = require("../../deps.js");
class TcpProvider {
    listen(opts) {
        const server = (0, net_1.createServer)();
        server.listen(opts.port);
        return new TcpListener(server);
    }
    connect(opts) {
        // console.log("creating connection", opts);
        const socket = (0, net_1.createConnection)(opts);
        return Promise.resolve(new TcpConn(socket));
    }
}
exports.TcpProvider = TcpProvider;
class TcpListener {
    constructor(server) {
        Object.defineProperty(this, "server", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "connQueue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new deps_js_1.AsyncQueue()
        });
        this.server = server;
        this.server.on("connection", (socket) => {
            this.connQueue.push(socket);
        });
    }
    close() {
        this.server.close();
    }
    async *[Symbol.asyncIterator]() {
        for await (const socket of this.connQueue) {
            yield new TcpConn(socket);
        }
    }
}
exports.TcpListener = TcpListener;
// A thing where you make a request for some number of bytes.
class SocketReader {
    constructor(socket) {
        Object.defineProperty(this, "promiseQueue", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "socket", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.socket = socket;
        socket.on("readable", () => {
            this.runQueue();
        });
        socket.on("end", () => {
            this.flush();
        });
    }
    runQueue() {
        while (this.promiseQueue.length > 0) {
            const item = this.promiseQueue[0];
            const bytes = this.socket.read(item.desiredLength);
            if (bytes) {
                //console.log("resolved bytes", bytes.byteLength);
                item.promise.resolve(bytes);
                this.promiseQueue.shift();
            }
            else {
                break;
            }
        }
    }
    flush() {
        const [head] = this.promiseQueue;
        const bytes = this.socket.read(undefined);
        if (bytes) {
            // console.log("flushed", bytes.byteLength);
            head.promise.resolve(bytes);
        }
        else {
            for (const item of this.promiseQueue) {
                // item.promise.reject("No data");
            }
        }
    }
    getBytes(len) {
        // Adds a promise to a chain
        const promise = (0, deps_js_1.deferred)();
        this.promiseQueue.push({
            desiredLength: len || undefined,
            promise,
        });
        this.runQueue();
        return promise;
    }
}
class TcpConn {
    constructor(socket) {
        Object.defineProperty(this, "socket", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "writable", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        //  isReady = deferred();
        Object.defineProperty(this, "socketReader", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: Math.random()
        });
        this.socket = socket;
        this.socketReader = new SocketReader(socket);
        socket.on("error", () => {
            this.writable.abort();
            this.readable.cancel();
        });
        /*
        if (socket.readyState === "open") {
          this.isReady.resolve();
        }
    
        socket.on("ready", () => {
          this.isReady.resolve();
        });
        */
        // const isReady = (() => this.isReady).bind(this);
        this.writable = new dntShim.WritableStream({
            async write(chunk) {
                //   await isReady();
                const hasWritten = (0, deps_js_1.deferred)();
                socket.write(chunk, () => {
                    hasWritten.resolve();
                });
                await hasWritten;
            },
            close() {
                socket.destroy();
            },
            abort() {
                socket.destroy();
            },
        });
    }
    // This is a getter (rather than constructed when TcpProvider is instantiated) because we want the controller to only start pulling when the readable is called for.
    get readable() {
        const getBytes = this.socketReader.getBytes.bind(this.socketReader);
        const socket = this.socket;
        //  const isReady = (() => this.isReady).bind(this);
        return new dntShim.ReadableStream({
            async pull(controller) {
                // await isReady();
                const bytes = await getBytes();
                if (bytes) {
                    controller.enqueue(bytes);
                }
            },
            cancel() {
                socket.destroy();
            },
        });
    }
    async read(bytes) {
        //    await this.isReady;
        const result = await this.socketReader.getBytes(bytes.byteLength);
        bytes.set(result, 0);
        return result.byteLength;
    }
    async write(bytes) {
        //   await this.isReady;
        const wrotePromise = (0, deps_js_1.deferred)();
        this.socket.write(bytes, () => {
            wrotePromise.resolve();
        });
        return wrotePromise;
    }
    close() {
        return this.socket.destroy();
    }
    get remoteAddr() {
        return {
            hostname: this.socket.remoteAddress || "Destroyed",
            port: this.socket.remotePort || 0,
        };
    }
}
