import { AsyncQueue } from "../../deps.js";
import { Syncer } from "../syncer/syncer.js";
import { ISyncPartner, SyncAppetite } from "../syncer/syncer_types.js";
import { DiscoveryService, DiscoveryServiceEvent } from "./types.js";
import { ITcpConn } from "../tcp/types.js";
type DiscoveryLANOpts = {
    /** The name we wish to use to identify ourselves to other peers on the network. */
    name: string;
    /** Whether to advertise our presence on the network or not. Defaults to true. */
    advertise?: boolean;
    /** The port to listen for incoming connections on. Defaults to 17171. */
    port?: number;
};
/** A discovery service for finding peers on the local network, to be used with `Peer.discover`.
 */
export declare class DiscoveryLAN implements DiscoveryService {
    private abortController;
    private multicastInterface;
    private tcpProvider;
    private sessions;
    private serviceNames;
    private eventQueue;
    constructor(opts: DiscoveryLANOpts);
    get events(): AsyncQueue<DiscoveryServiceEvent>;
    stop(): void;
}
type LANSessionOpts = {
    initiator?: {
        appetite: SyncAppetite;
        port: number;
    };
    target: {
        hostname: string;
        name?: string;
    };
    ourPort: number;
    onComplete: () => void;
};
export declare class LANSession {
    initiator: {
        appetite: SyncAppetite;
    } | undefined;
    description: string;
    private tcpProvider;
    private keypair;
    private derivedSecret;
    private onComplete;
    private targetPort;
    private ourPort;
    partner: import("../../deps.js").Deferred<ISyncPartner<ITcpConn>>;
    syncer: import("../../deps.js").Deferred<Syncer<ITcpConn, unknown>>;
    constructor(opts: LANSessionOpts);
    addConn(conn: ITcpConn): Promise<void>;
    addKeyExchangeConn(conn: ITcpConn): Promise<void>;
    addMessageConn(conn: ITcpConn): Promise<void>;
    addAttachmentConn(conn: ITcpConn): Promise<void>;
}
export {};
